/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.ui;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import benten.ui.messages.AbstractBentenUiPluginMessages;

/**
 * Benten UI プラグインの基底クラス。
 *
 * <UL>
 * <LI>Benten で使用する UI 系プラグインの基底クラスです。
 * </UL>
 *
 * @author KASHIHARA Shinji
 */
public abstract class AbstractBentenUiPlugin extends AbstractUIPlugin {

	/**
	 * Benten UI プラグインの基底クラスのためのメッセージ。
	 */
	protected static final AbstractBentenUiPluginMessages fMsg = new AbstractBentenUiPluginMessages();

	/**
	 * デバッグするかどうか。
	 *
	 * <UL>
	 * <LI>Benten 開発者向けのデバッグ・フラグです。
	 * <LI>Java 起動オプションで -Dbenten.debug=true を指定することにより有効となります。
	 * </UL>
	 */
	public static final boolean IS_DEBUG = Boolean.valueOf(System.getProperty("benten.debug")); //$NON-NLS-1$

	/**
	 * プラグイン id の取得。
	 *
	 * <UL>
	 * <LI>Benten プロジェクトではプラグイン id はパッケージ名と同一です。
	 * </UL>
	 *
	 * @return プラグイン id
	 */
	public final String getPluginId() {
		return getClass().getPackage().getName();
	}

	/**
	 * 指定されたメッセージのログ出力。
	 *
	 * <UL>
	 * <LI>引数の文字列表現が Eclipse のログに出力されます。
	 * <LI>引数が例外の場合は上記に加え、エラー・ダイアログを表示します。
	 * </UL>
	 *
	 * @param message ログ出力するメッセージ
	 */
	public void log(final Object message) {
		if (message instanceof Throwable) {
			final Throwable e = (Throwable) message;
			getLog().log(new Status(IStatus.WARNING, getPluginId(), -1, e.toString(), e));
			if (IS_DEBUG) {
				e.printStackTrace();
			}
			Display.getDefault().asyncExec(new Runnable() {
				public void run() {
					final UiStatus s = new UiStatus(IStatus.ERROR, e.getMessage(), e);
					UiPlugin.openDialog(fMsg.getMsg001(), s);
				}
			});
		} else {
			final String s = message == null ? "" : message.toString(); //$NON-NLS-1$
			getLog().log(new Status(IStatus.WARNING, getPluginId(), s));
			if (IS_DEBUG) {
				System.out.println(message);
			}
		}
	}

	/**
	 * このプラグイン配下のファイルを取得。
	 *
	 * <UL>
	 * <LI>指定する path の先頭は "/" である必要があります。
	 * </UL>
	 *
	 * @param path パス
	 * @return ファイル。存在しない場合やアクセスできない場合は null。
	 */
	public File getFile(final String path) {
		try {
			final URL entry = getBundle().getEntry(path);
			return new File(FileLocator.resolve(entry).getPath());
		} catch (final IOException e) {
			log(e);
			return null;
		}
	}

	/**
	 * イメージの取得。
	 *
	 * <UL>
	 * <LI>一度取得したイメージは Eclipse のイメージ・レジストリーで管理され、適切に破棄されます。
	 * </UL>
	 *
	 * @param path パス
	 * @return イメージ。存在しない場合は null。
	 */
	public Image getImage(final String path) {
		Image image = getImageRegistry().get(path);
		if (image == null) {
			final ImageDescriptor descriptor = imageDescriptorFromPlugin(getPluginId(), path);
			if (descriptor != null) {
				getImageRegistry().put(path, image = descriptor.createImage());
			}
		}
		return image;
	}
}
